==> cx88-1.0.2.tar.gz: cx88-1.0.2/tests/cx88_audio_client.cpp <==
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/soundcard.h>
#include "cx88_audio_ioctl.h"
#include "tuner_devnode_device.h"
#include "lgh064f.h"
#include "dtt7612.h"

extern char *optarg;

void capture(int fd, cx88_audio_config &config, char *outfile, uint32_t time_s)
{
   int channels = 1;         // 0=mono 1=stereo
   int format = AFMT_S16_LE;
   int rate = 48000;
   int outfd = open(outfile, O_WRONLY);
   if (outfd < 0)
   {
      printf("Error opening output file\n");
      return;  
   }

   /* Tell the sound card that the sound about to be played is stereo. 0=mono 1=stereo */
   if (ioctl(outfd, SNDCTL_DSP_STEREO, &channels) == -1)
   {
      printf("Error during SNDCTL_DSP_STEREO\n");
      close(outfd);
      return;
   }

   if (ioctl(outfd, SNDCTL_DSP_SETFMT, &format) == -1)
   {
      printf("Error during SNDCTL_DSP_SETFMT\n");
      close(outfd);
      return;
   }
   
   /* Set the DSP playback rate, sampling rate of the raw PCM audio */
   if (ioctl(outfd, SNDCTL_DSP_SPEED, &rate) == -1 )
   {
      printf("Error during SNDCTL_DSP_SPEED\n");
      close(outfd);
      return;
   }

   void *buf = config.bufconfig.in_userbuf;
   uint32_t segsize = config.out_samples_per_buffer * CX88_AUDIO_SAMPLE_SIZE;
   uint32_t bufcapacity = config.in_num_samples * CX88_AUDIO_SAMPLE_SIZE;
   if (config.in_buftype == CX88_AUDIO_BUF_KERNEL)
   {
      buf = mmap(NULL, config.bufconfig.out_kernelbuf.out_mmap_size, PROT_READ, MAP_SHARED, fd, 0);
      if (buf == MAP_FAILED)
      {
         perror("Error mmap'ing device:");
         close(outfd);
         return;
      }
      segsize = config.bufconfig.out_kernelbuf.out_buffer_size;
      bufcapacity = config.bufconfig.out_kernelbuf.out_mmap_size;
   }
   int error = ioctl(fd, CX88_AUDIO_CAPTURE);
   if (error)
   {
      printf("Error during capture ioctl\n");  
   }
   else
   {
      cx88_audio_wait wait;
      wait.in_timeout_ms = 0;
      time_t start_time = time(NULL);
      do
      {
         error = ioctl(fd, CX88_AUDIO_WAIT, &wait);
         if (error)
         {
            printf("Error during wait ioctl\n");
            close(outfd);
            ioctl(fd, CX88_AUDIO_STOP);
            return;
         }
         if ((wait.out_status & CX88_AUDIO_SYNC_ERROR) == CX88_AUDIO_SYNC_ERROR)
         {
            printf("Fatal error: DMA sync error\n");
         }        
         if ((wait.out_status & CX88_AUDIO_BUS_ERROR) == CX88_AUDIO_BUS_ERROR)
         {
            printf("Fatal error: PCI bus error\n");
         }
         if ((wait.out_status & CX88_AUDIO_DMA_PROGRAM) == CX88_AUDIO_DMA_PROGRAM)
         {
            printf("Fatal error: DMA program error\n");
         }
         if ((wait.out_status & CX88_AUDIO_WAIT_ABORTED) == CX88_AUDIO_WAIT_ABORTED)
         {
            printf("Fatal error: Wait aborted\n");
         }
         if ((wait.out_status & CX88_AUDIO_CAP_TIMEOUT) == CX88_AUDIO_CAP_TIMEOUT)
         {
            printf("Nonfatal error: Capture timed out\n");
         }
         if ((wait.out_status & CX88_AUDIO_BERTHRESH) == CX88_AUDIO_BERTHRESH)
         {
            printf("Nonfatal error: BER threshold exceeded\n");
         }
         if ((wait.out_status & CX88_AUDIO_OVERFLOW) == CX88_AUDIO_OVERFLOW)
         {
            printf("Nonfatal error: overflow detected\n");
         }
         if (wait.out_status & CX88_AUDIO_CAP_FATAL)
         {
            break;
         }
         if (wait.out_buffer_available == 0 || wait.out_buffer_available > config.out_num_buffers)
         {
            printf("No buffer available\n");
            break;
         }
         uint32_t bufoffset = (wait.out_buffer_available - 1) * segsize;
         uint32_t bufsize = bufcapacity - bufoffset;
         char *bufptr = (char *)buf + bufoffset;
         if ((wait.out_buffer_available < config.out_num_buffers) &&
             (bufsize > (config.out_samples_per_buffer * CX88_AUDIO_SAMPLE_SIZE)))
         {
            bufsize = config.out_samples_per_buffer * CX88_AUDIO_SAMPLE_SIZE;
         }
         uint32_t count = 0, writecount = 0;
         while (count < bufsize)
         {
            writecount = bufsize - count;
            ssize_t sz2 = write(outfd, bufptr, writecount);
            if (sz2 < 0)
            {
               break;
            }
            bufptr += sz2;
            count += sz2;
         }
      }
      while ((unsigned int) (time(NULL) - start_time) < time_s);
      printf("Issuing STOP ioctl\n");
      ioctl(fd, CX88_AUDIO_STOP);
      printf("Finished STOP\n");
      if (ioctl(outfd, SNDCTL_DSP_SYNC) == -1)
      {
         printf("Error during SNDCTL_DSP_SYNC\n");
      }
      close(outfd);
   }
   if (config.in_buftype == CX88_AUDIO_BUF_KERNEL)
   {
      printf("Preparing to munmap %d\n", config.bufconfig.out_kernelbuf.out_mmap_size);
      munmap(buf, config.bufconfig.out_kernelbuf.out_mmap_size);  
      printf("Finished munmapping\n"); 
   }
}

int main(int argc, char **argv)
{
   cx88_audio_i2c_config i2config;
   uint32_t id;
   int optch = -1;
   i2config.in_i2c_addrs[0] = 0x61;
   i2config.in_num_i2c_devices = 1;
   int fd = open(argv[1], O_RDWR);
   int error = ioctl(fd, CX88_AUDIO_CARD_ID, &id);
   if (error)
   {
      printf("Error during card ID ioctl\n");
      return 0;
   }
   printf("Card ID = 0x%x\n", id);
   error = ioctl(fd, CX88_AUDIO_I2C_CONFIG, &i2config);
   if (error)
   {
      printf("Error during i2c config ioctl\n");
      return 0;
   }
   printf("Tuner: %s\n", i2config.out_i2c_devnodes[0]);
   cx88_audio_format fmt;
   fmt.in_audio_format = CX88_AUDIO_FMT_BTSC;
   fmt.in_stereo_format = CX88_AUDIO_STEREO_AUTO;
   fmt.in_second_lang = 0;
   fmt.in_input = CX88_AUDIO_INPUT_BCAST;
   fmt.in_i2s_config = 0;
   error = ioctl(fd, CX88_AUDIO_FORMAT, &fmt);
   if (error)
   {
      printf("Error during format ioctl\n");
      return 0;
   }
   char tuner_devnode[CX88_AUDIO_MAX_DEVNODE_NAME + 5];
   sprintf(tuner_devnode, "/dev/%s", i2config.out_i2c_devnodes[0]);
   avb_channel channel;
   channel.format = AVB_FORMAT_NTSC_M;
   channel.frequency_hz = 641000000;
   channel.bandwidth_hz = 6000000;
   tuner_config conf;
   error = conf.load_file("tuner_config");
   if (error)
   {
      printf("Error loading tuner config file\n");
      close(fd);
      return 0;
   }
   tuner_devnode_device tuner_dev(conf, tuner_devnode, error);
   if (error)
   {
      printf("Error creating tuner devnode\n");
      close(fd);
      return 0;
   }
   pll_driver *tuner_driver = NULL;
   switch (id)
   {
      case 0x70633000:
         tuner_driver = new dtt7612(conf, tuner_dev);
         break;
      case 0x70635500:
         tuner_driver = new lgh064f(conf, tuner_dev);
         break;
      default:
         printf("Unrecognized card type: %d\n", id);
         break;
   }
   if (tuner_driver == NULL)
   {
      printf("Unable to create tuner driver\n");
      close(fd);
      return 0;
   }
   tuner_driver->set_channel(channel);
   printf("Ready to start tuner\n");
   error = tuner_driver->start(10000);
   if (error)
   {
      printf("Error starting tuner\n");
      close(fd);
      return 0;
   }
   printf("Ready for capture\n");
   bool pcm_capture = true;
   char *outfile = NULL;
   uint32_t time_s = 0;
   cx88_audio_config config;
   while ((optch = getopt(argc - 1, &(argv[1]), "u:k:f:t:")) != -1)
   {
      pcm_capture = false;
      switch (optch)
      {
         case 't':
            time_s = strtoul(optarg, NULL, 10);
            break;
         case 'f':
            outfile = strdup(optarg);
            break;
         case 'u':
         case 'k':
            config.in_num_samples = strtoul(optarg, NULL, 10);
            if (optch == 'u')
            {
               config.in_buftype = CX88_AUDIO_BUF_USER;  
               config.bufconfig.in_userbuf = malloc(config.in_num_samples * CX88_AUDIO_SAMPLE_SIZE);
            }
            else if (optch == 'k')
            {
               config.in_buftype = CX88_AUDIO_BUF_KERNEL;
            }
            error = ioctl(fd, CX88_AUDIO_CONFIG, &config);
            if (error)
            {
               printf("Error during config ioctl\n");
               close(fd);
               return 0;  
            }
            break;
      }
   }
   if (pcm_capture)
   {
      int ch;
      while ((ch = getchar()) != 27)
      {
         switch(ch)
         {
            case 's':
            {
               cx88_audio_status stat;
               error = ioctl(fd, CX88_AUDIO_GET_STATUS, &stat);
               if (error)
               {
                  printf("Error during status ioctl\n");
                  continue;
               }
               printf("Status:\n\tVolume: %d:%d\n\tMute: %d\n\tCarrier: %d\n\tSignal: %d\n\n",
                  stat.vol_left, stat.vol_right, stat.mute, stat.carrier, stat.signal);
               break;
            }
            case 'm':
               error = ioctl(fd, CX88_AUDIO_MUTE);
               if (error)
               {
                  printf("Error during mute ioctl\n");
                  continue;
               }
               break;
         }
      }
   }
   else
   {
      capture(fd, config, outfile, time_s);
   }
   delete tuner_driver;
   close(fd);
   free(outfile);
   return 0;
}

==> cx88-1.1.0.tar.gz: cx88-1.1.0/tests/cx88_audio_client.cpp <==
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/soundcard.h>
#include "cx88_audio_ioctl.h"
#include "tuner_devnode_device.h"
#include "lgh064f.h"
#include "dtt7612.h"

extern char *optarg;

void status(int fd)
{
   int error = 0;
   cx88_audio_status stat;
   error = ioctl(fd, CX88_AUDIO_GET_STATUS, &stat);
   if (error)
   {
      printf("Error during status ioctl\n");
      return;
   }
   printf("Status:\n\tVolume: %d:%d\n\tMute: %d\n\tCarrier: %d\n\tSignal: %d\n\n",
      stat.out_vol_left, stat.out_vol_right, stat.out_mute, stat.out_carrier, stat.out_signal);
}

void capture(int fd, cx88_audio_config &config, char *outfile, uint32_t time_s)
{
   int channels = 1;         // 0=mono 1=stereo
   int format = AFMT_S16_LE;
   int rate = 48000;
   int outfd = open(outfile, O_WRONLY);
   if (outfd < 0)
   {
      printf("Error opening output file\n");
      return;  
   }

   /* Tell the sound card that the sound about to be played is stereo. 0=mono 1=stereo */
   if (ioctl(outfd, SNDCTL_DSP_STEREO, &channels) == -1)
   {
      printf("Error during SNDCTL_DSP_STEREO\n");
      close(outfd);
      return;
   }

   if (ioctl(outfd, SNDCTL_DSP_SETFMT, &format) == -1)
   {
      printf("Error during SNDCTL_DSP_SETFMT\n");
      close(outfd);
      return;
   }
   
   /* Set the DSP playback rate, sampling rate of the raw PCM audio */
   if (ioctl(outfd, SNDCTL_DSP_SPEED, &rate) == -1 )
   {
      printf("Error during SNDCTL_DSP_SPEED\n");
      close(outfd);
      return;
   }

   void *buf = config.bufconfig.in_userbuf;
   uint32_t segsize = config.out_samples_per_buffer * CX88_AUDIO_SAMPLE_SIZE;
   uint32_t bufcapacity = config.in_num_samples * CX88_AUDIO_SAMPLE_SIZE;
   if (config.in_buftype == CX88_AUDIO_BUF_KERNEL)
   {
      buf = mmap(NULL, config.bufconfig.out_kernelbuf.out_mmap_size, PROT_READ, MAP_SHARED, fd, 0);
      if (buf == MAP_FAILED)
      {
         perror("Error mmap'ing device:");
         close(outfd);
         return;
      }
      segsize = config.bufconfig.out_kernelbuf.out_buffer_size;
      bufcapacity = config.bufconfig.out_kernelbuf.out_mmap_size;
   }
   int error = ioctl(fd, CX88_AUDIO_CAPTURE);
   if (error)
   {
      printf("Error during capture ioctl\n");  
   }
   else
   {
      status(fd);
      cx88_audio_wait wait;
      wait.in_timeout_ms = 0;
      time_t start_time = time(NULL);
      do
      {
         error = ioctl(fd, CX88_AUDIO_WAIT, &wait);
         if (error)
         {
            printf("Error during wait ioctl\n");
            close(outfd);
            ioctl(fd, CX88_AUDIO_STOP);
            return;
         }
         if ((wait.out_status & CX88_AUDIO_SYNC_ERROR) == CX88_AUDIO_SYNC_ERROR)
         {
            printf("Fatal error: DMA sync error\n");
         }        
         if ((wait.out_status & CX88_AUDIO_BUS_ERROR) == CX88_AUDIO_BUS_ERROR)
         {
            printf("Fatal error: PCI bus error\n");
         }
         if ((wait.out_status & CX88_AUDIO_DMA_PROGRAM) == CX88_AUDIO_DMA_PROGRAM)
         {
            printf("Fatal error: DMA program error\n");
         }
         if ((wait.out_status & CX88_AUDIO_WAIT_ABORTED) == CX88_AUDIO_WAIT_ABORTED)
         {
            printf("Fatal error: Wait aborted\n");
         }
         if ((wait.out_status & CX88_AUDIO_CAP_TIMEOUT) == CX88_AUDIO_CAP_TIMEOUT)
         {
            printf("Nonfatal error: Capture timed out\n");
         }
         if ((wait.out_status & CX88_AUDIO_BERTHRESH) == CX88_AUDIO_BERTHRESH)
         {
            printf("Nonfatal error: BER threshold exceeded\n");
         }
         if ((wait.out_status & CX88_AUDIO_OVERFLOW) == CX88_AUDIO_OVERFLOW)
         {
            printf("Nonfatal error: overflow detected\n");
         }
         if (wait.out_status & CX88_AUDIO_CAP_FATAL)
         {
            break;
         }
         if (wait.out_buffer_available == 0 || wait.out_buffer_available > config.out_num_buffers)
         {
            printf("No buffer available\n");
            break;
         }
         uint32_t bufoffset = (wait.out_buffer_available - 1) * segsize;
         uint32_t bufsize = bufcapacity - bufoffset;
         char *bufptr = (char *)buf + bufoffset;
         if ((wait.out_buffer_available < config.out_num_buffers) &&
             (bufsize > (config.out_samples_per_buffer * CX88_AUDIO_SAMPLE_SIZE)))
         {
            bufsize = config.out_samples_per_buffer * CX88_AUDIO_SAMPLE_SIZE;
         }
         uint32_t count = 0, writecount = 0;
         while (count < bufsize)
         {
            writecount = bufsize - count;
            ssize_t sz2 = write(outfd, bufptr, writecount);
            if (sz2 < 0)
            {
               break;
            }
            bufptr += sz2;
            count += sz2;
         }
      }
      while ((unsigned int) (time(NULL) - start_time) < time_s);
      status(fd);
      printf("Issuing STOP ioctl\n");
      ioctl(fd, CX88_AUDIO_STOP);
      printf("Finished STOP\n");
      if (ioctl(outfd, SNDCTL_DSP_SYNC) == -1)
      {
         printf("Error during SNDCTL_DSP_SYNC\n");
      }
      close(outfd);
   }
   if (config.in_buftype == CX88_AUDIO_BUF_KERNEL)
   {
      printf("Preparing to munmap %d\n", config.bufconfig.out_kernelbuf.out_mmap_size);
      munmap(buf, config.bufconfig.out_kernelbuf.out_mmap_size);  
      printf("Finished munmapping\n"); 
   }
}

int main(int argc, char **argv)
{
   cx88_common_i2c_config i2config;
   uint32_t id;
   int optch = -1;
   i2config.in_i2c_addrs[0] = 0x61;
   i2config.in_num_i2c_devices = 1;
   int fd = open(argv[1], O_RDWR);
   int error = ioctl(fd, CX88_COMMON_CARD_ID, &id);
   if (error)
   {
      printf("Error during card ID ioctl\n");
      return 0;
   }
   printf("Card ID = 0x%x\n", id);
   error = ioctl(fd, CX88_COMMON_I2C_CONFIG, &i2config);
   if (error)
   {
      printf("Error during i2c config ioctl\n");
      return 0;
   }
   printf("Tuner: %s\n", i2config.out_i2c_devnodes[0]);
   cx88_audio_format fmt;
   fmt.in_audio_format = CX88_AUDIO_FMT_BTSC;
   fmt.in_stereo_format = CX88_AUDIO_STEREO_AUTO;
   fmt.in_second_lang = 0;
   fmt.in_input = CX88_AUDIO_INPUT_BCAST;
   fmt.in_i2s_config = 0;
   error = ioctl(fd, CX88_AUDIO_FORMAT, &fmt);
   if (error)
   {
      printf("Error during format ioctl\n");
      return 0;
   }
   char tuner_devnode[CX88_COMMON_MAX_DEVNODE_NAME + 5];
   sprintf(tuner_devnode, "/dev/%s", i2config.out_i2c_devnodes[0]);
   avb_channel channel;
   channel.video_format = AVB_VIDEO_FMT_NTSC_M;
   channel.audio_format = AVB_AUDIO_FMT_AUTO;
   channel.frequency_hz = 641000000;
   channel.bandwidth_hz = 6000000;
   tuner_config conf;
   error = conf.load_file("tuner_config");
   if (error)
   {
      printf("Error loading tuner config file\n");
      close(fd);
      return 0;
   }
   tuner_devnode_device tuner_dev(conf, tuner_devnode, error);
   if (error)
   {
      printf("Error creating tuner devnode\n");
      close(fd);
      return 0;
   }
   pll_driver *tuner_driver = NULL;
   switch (id)
   {
      case 0x70633000:
         tuner_driver = new dtt7612(conf, tuner_dev);
         break;
      case 0x70635500:
         tuner_driver = new lgh064f(conf, tuner_dev);
         break;
      default:
         printf("Unrecognized card type: %d\n", id);
         break;
   }
   if (tuner_driver == NULL)
   {
      printf("Unable to create tuner driver\n");
      close(fd);
      return 0;
   }
   tuner_driver->set_channel(channel);
   printf("Ready to start tuner\n");
   error = tuner_driver->start(10000);
   if (error)
   {
      printf("Error starting tuner\n");
      close(fd);
      return 0;
   }
   printf("Ready for capture\n");
   bool pcm_capture = true;
   char *outfile = NULL;
   uint32_t time_s = 0;
   cx88_audio_config config;
   while ((optch = getopt(argc - 1, &(argv[1]), "u:k:f:t:")) != -1)
   {
      pcm_capture = false;
      switch (optch)
      {
         case 't':
            time_s = strtoul(optarg, NULL, 10);
            break;
         case 'f':
            outfile = strdup(optarg);
            break;
         case 'u':
         case 'k':
            config.in_num_samples = strtoul(optarg, NULL, 10);
            if (optch == 'u')
            {
               config.in_buftype = CX88_AUDIO_BUF_USER;  
               config.bufconfig.in_userbuf = malloc(config.in_num_samples * CX88_AUDIO_SAMPLE_SIZE);
            }
            else if (optch == 'k')
            {
               config.in_buftype = CX88_AUDIO_BUF_KERNEL;
            }
            error = ioctl(fd, CX88_AUDIO_CONFIG, &config);
            if (error)
            {
               printf("Error during config ioctl\n");
               close(fd);
               return 0;  
            }
            break;
      }
   }
   if (pcm_capture)
   {
      int ch;
      while ((ch = getchar()) != 27)
      {
         switch(ch)
         {
            case 's':
            {
               status(fd);
               break;
            }
            case 'm':
               error = ioctl(fd, CX88_AUDIO_MUTE);
               if (error)
               {
                  printf("Error during mute ioctl\n");
                  continue;
               }
               break;
         }
      }
   }
   else
   {
      capture(fd, config, outfile, time_s);
   }
   delete tuner_driver;
   close(fd);
   free(outfile);
   return 0;
}
